"""Convert ADIF ADI/ADX content to dictionary and vice versa"""

from adif_file.__version__ import __version__ as __version_str__

__author_name__ = 'Andreas Schawo, DF1ASC'
__author_email__ = 'andreas@schawo.de'
__copyright__ = 'Copyright 2023 by Andreas Schawo,licensed under CC BY-SA 4.0'
__proj_name__ = 'PyADIF-File'
__version__ = __version_str__[1:].split('-')[0]
__description__ = 'Convert ADIF ADI/ADX content to dictionary and vice versa'
